"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

const axios = require(`axios`);

const _ = require(`lodash`);

const _require = require(`gatsby-source-filesystem`),
      createRemoteFileNode = _require.createRemoteFileNode;

const _require2 = require(`url`),
      URL = _require2.URL;

const _require3 = require(`./normalize`),
      nodeFromData = _require3.nodeFromData;

const asyncPool = require(`tiny-async-pool`);

exports.sourceNodes =
/*#__PURE__*/
function () {
  var _ref = (0, _asyncToGenerator2.default)(function* ({
    actions,
    store,
    cache,
    createNodeId,
    createContentDigest,
    reporter
  }, {
    baseUrl,
    apiBase,
    basicAuth,
    filters,
    headers,
    params,
    concurrentFileRequests
  }) {
    const createNode = actions.createNode;
    const drupalFetchActivity = reporter.activityTimer(`Fetch data from Drupal`);
    const downloadingFilesActivity = reporter.activityTimer(`Remote file download`); // Default apiBase to `jsonapi`

    apiBase = apiBase || `jsonapi`; // Default concurrentFileRequests to `20`

    const namespacedCreateNodeId = id => createNodeId(`${baseUrl}/${id}`);

    concurrentFileRequests = concurrentFileRequests || 20; // Touch existing Drupal nodes so Gatsby doesn't garbage collect them.
    // _.values(store.getState().nodes)
    // .filter(n => n.internal.type.slice(0, 8) === `drupal__`)
    // .forEach(n => touchNode({ nodeId: n.id }))
    // Fetch articles.
    // console.time(`fetch Drupal data`)

    reporter.info(`Starting to fetch data from Drupal`); // TODO restore this
    // let lastFetched
    // if (
    // store.getState().status.plugins &&
    // store.getState().status.plugins[`gatsby-source-drupal`]
    // ) {
    // lastFetched = store.getState().status.plugins[`gatsby-source-drupal`].status
    // .lastFetched
    // }

    drupalFetchActivity.start();
    const data = yield axios.get(`${baseUrl}/${apiBase}`, {
      auth: basicAuth,
      headers,
      params
    });
    const allData = yield Promise.all(_.map(data.data.links,
    /*#__PURE__*/
    function () {
      var _ref2 = (0, _asyncToGenerator2.default)(function* (url, type) {
        if (type === `self`) return;
        if (!url) return;
        if (!type) return;

        const getNext =
        /*#__PURE__*/
        function () {
          var _ref3 = (0, _asyncToGenerator2.default)(function* (url, data = []) {
            if (typeof url === `object`) {
              // url can be string or object containing href field
              url = url.href; // Apply any filters configured in gatsby-config.js. Filters
              // can be any valid JSON API filter query string.
              // See https://www.drupal.org/docs/8/modules/jsonapi/filtering

              if (typeof filters === `object`) {
                if (filters.hasOwnProperty(type)) {
                  url = url + `?${filters[type]}`;
                }
              }
            }

            let d;

            try {
              d = yield axios.get(url, {
                auth: basicAuth,
                headers,
                params
              });
            } catch (error) {
              if (error.response && error.response.status == 405) {
                // The endpoint doesn't support the GET method, so just skip it.
                return [];
              } else {
                console.error(`Failed to fetch ${url}`, error.message);
                console.log(error.data);
                throw error;
              }
            }

            data = data.concat(d.data.data);

            if (d.data.links.next) {
              data = yield getNext(d.data.links.next, data);
            }

            return data;
          });

          return function getNext(_x5) {
            return _ref3.apply(this, arguments);
          };
        }();

        const data = yield getNext(url);
        const result = {
          type,
          data // eslint-disable-next-line consistent-return

        };
        return result;
      });

      return function (_x3, _x4) {
        return _ref2.apply(this, arguments);
      };
    }()));
    drupalFetchActivity.end(); // Make list of all IDs so we can check against that when creating
    // relationships.

    const ids = {};

    _.each(allData, contentType => {
      if (!contentType) return;

      _.each(contentType.data, datum => {
        ids[datum.id] = true;
      });
    }); // Create back references


    const backRefs = {};
    /**
     * Adds back reference to linked entity, so we can later
     * add node link.
     */

    const addBackRef = (linkedId, sourceDatum) => {
      if (ids[linkedId]) {
        if (!backRefs[linkedId]) {
          backRefs[linkedId] = [];
        }

        backRefs[linkedId].push({
          id: sourceDatum.id,
          type: sourceDatum.type
        });
      }
    };

    _.each(allData, contentType => {
      if (!contentType) return;

      _.each(contentType.data, datum => {
        if (datum.relationships) {
          _.each(datum.relationships, (v, k) => {
            if (!v.data) return;

            if (_.isArray(v.data)) {
              v.data.forEach(data => addBackRef(data.id, datum));
            } else {
              addBackRef(v.data.id, datum);
            }
          });
        }
      });
    }); // Process nodes


    const nodes = [];

    _.each(allData, contentType => {
      if (!contentType) return;

      _.each(contentType.data, datum => {
        const node = nodeFromData(datum, namespacedCreateNodeId);
        node.relationships = {}; // Add relationships

        if (datum.relationships) {
          _.each(datum.relationships, (v, k) => {
            if (!v.data) return;

            if (_.isArray(v.data) && v.data.length > 0) {
              // Create array of all ids that are in our index
              node.relationships[`${k}___NODE`] = _.compact(v.data.map(data => ids[data.id] ? namespacedCreateNodeId(data.id) : null));
            } else if (ids[v.data.id]) {
              node.relationships[`${k}___NODE`] = namespacedCreateNodeId(v.data.id);
            }
          });
        } // Add back reference relationships.
        // Back reference relationships will need to be arrays,
        // as we can't control how if node is referenced only once.


        if (backRefs[datum.id]) {
          backRefs[datum.id].forEach(ref => {
            if (!node.relationships[`${ref.type}___NODE`]) {
              node.relationships[`${ref.type}___NODE`] = [];
            }

            node.relationships[`${ref.type}___NODE`].push(namespacedCreateNodeId(ref.id));
          });
        }

        if (_.isEmpty(node.relationships)) {
          delete node.relationships;
        }

        node.internal.contentDigest = createContentDigest(node);
        nodes.push(node);
      });
    });

    reporter.info(`Downloading remote files from Drupal`);
    downloadingFilesActivity.start(); // Download all files (await for each pool to complete to fix concurrency issues)

    yield asyncPool(concurrentFileRequests, nodes,
    /*#__PURE__*/
    function () {
      var _ref4 = (0, _asyncToGenerator2.default)(function* (node) {
        // If we have basicAuth credentials, add them to the request.
        const auth = typeof basicAuth === `object` ? {
          htaccess_user: basicAuth.username,
          htaccess_pass: basicAuth.password
        } : {};
        let fileNode = null;
        let fileUrl = ``;
        let url = {};

        if (node.internal.type === `files` || node.internal.type === `file__file`) {
          fileUrl = node.url; // If node.uri is an object

          if (typeof node.uri === `object`) {
            // Support JSON API 2.x file URI format https://www.drupal.org/node/2982209
            fileUrl = node.uri.url;
          } // Resolve w/ baseUrl if node.uri isn't absolute.


          url = new URL(fileUrl, baseUrl); // Create the remote file from the given node

          try {
            fileNode = yield createRemoteFileNode({
              url: url.href,
              store,
              cache,
              createNode,
              createNodeId,
              parentNodeId: node.id,
              auth
            });
          } catch (err) {
            reporter.error(err);
          } // If the fileNode exists set the node ID of the local file


          if (fileNode) {
            node.localFile___NODE = fileNode.id;
          }
        }
      });

      return function (_x6) {
        return _ref4.apply(this, arguments);
      };
    }());
    downloadingFilesActivity.end(); // Create each node

    for (var _i = 0; _i < nodes.length; _i++) {
      const node = nodes[_i];
      createNode(node);
    }
  });

  return function (_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();